
"""
Nom/Prénom: Heredero/Louis
Explications:

Une approche pour résoudre ce problème grâce à la programmation dynamique consiste
à se représenter les différentes séquences comme un arbre de décisions, où chaque nœud
représente une séquence, et chaque arête un coup supplémentaire
On peut alors parcourir l'arbre en profondeur (DFS) jusqu'à trouver une séquence
de longueur N et de valeur H (en élaguant les branches dépassant H ou N)
On peut également mémoïser un certain nombre de valeurs : en effet, pour une même
combinaison N, C, H, le résultat sera toujours le même. Cela optimise donc grandement
le calcul récursif puisque cela évite de calculer les états partagés (sous-branches
identiques)

Bonus :
Nous cherchons la valeur N la plus petite telle que:
  `computeNbrOfDifferentSequences(N, 5, 100) > 2^16`
"""

mem: dict[tuple[int, int, int], int] = {}

def computeNbrOfDifferentSequences(N: int, C: int, H: int) -> int:
    if N == 0:
        if H == 0:
            return 1
        return 0

    if H < 0:
        return 0

    key: tuple[int, int, int] = (N, C, H)
    if key in mem:
        return mem[key]

    total: int = 0
    for c in range(1, C + 1):
        total += computeNbrOfDifferentSequences(N - 1, C, H - c)

    mem[key] = total
    return total


def bonus():
    # Finding N2
    N1 = 100
    N2 = 1_000_000_000
    C = 5
    H = 100
    lim = 2 ** 16

    while computeNbrOfDifferentSequences(N2, C, H) <= lim:
        N2 *= 2
        print(f"*= 2 -> {N2}")

    print(f"{N2=}")

    # Finding N
    while N2 - N1 > 1:
        Nm = (N1 + N2) // 2
        value = computeNbrOfDifferentSequences(Nm, C, H)
        if value == lim:
            return Nm

        if value < lim:
            N1 = Nm
        else:
            N2 = Nm

    return N2

if __name__ == '__main__':
    print(computeNbrOfDifferentSequences(1, 6, 3) == 1)
    print(computeNbrOfDifferentSequences(2, 6, 7) == 6)
    print(bonus())
